#include "MP1051.h"
#include <avr/interrupt.h>
//######################################################################################################################## 
byte ACNT;		//     
byte BRGH;		//  
byte LED[4][2];		//   
byte IR_ADR, IR_DATA;	//     -
//######################################################################################################################## 
MP1051Class MP1051;
//######################################################################################################################## 
void MP1051Class::Init()
{	//    A0-A3 (PC0-PC3), A4-A5 (SDI, PC4-PC5), D2-D7 (PD2-PD6, CLK, LE, OE, SDO(2), IR)
	DDRC|=0x3F;
	PORTC|=0x0F;
	DDRD|=0x1C;
	DDRD&=0x1F;
	PORTD&=0xFB;  //  CLK
	PORTD&=0xF7;  //  LE
	PORTD|=0x90;  //  OE, IR
	for (byte I=0; I<4; I++)
		for (byte J=0; J<2; J++)
			LED[I][J]=0;
	//     
	TIMSK2&=~(1<<TOIE2);
	TCCR2A&=~((1<<WGM21)|(1<<WGM20));
	TCCR2B&=~(1<<WGM22);
	ASSR&=~(1<<AS2);
	TIMSK2&=~(1<<OCIE2A);
	TCCR2B|=(1<<CS21);
	TCCR2B&=~((1<<CS22) | (1<<CS20));
 	TCNT2=0xFF;
	TIMSK2|=(1<<TOIE2);
	ACNT=0;
	BRGH=255;
}
//######################################################################################################################## 
void MP1051Class::Set(byte D1, byte D2, byte D3, byte D4, byte D5, byte D6, byte D7, byte D8)
{	//     
	LED[0][0]=D1;	LED[0][1]=D2;
	LED[1][0]=D3;	LED[1][1]=D4;
	LED[2][0]=D5;	LED[2][1]=D6;
	LED[3][0]=D7;	LED[3][1]=D8;
}
//######################################################################################################################## 
void MP1051Class::Brightness(byte B)
{	//    0...32
	BRGH=B;
}
//######################################################################################################################## 
byte MP1051Class::IRWait(int T)
{	//	  T ms
	//	0 -     ,  1/2
	//	1 - RESET ,  
	//	2 - 
	int I;
	for (int I=0; I<T; I++)
		for (int J=0; J<10; J++)
			{ delayMicroseconds(100); if ((PIND&0x80)==0) goto IRWait_0; }
	return 0;
IRWait_0:
	//	 RESET 7...11ms
	for (I=0; I<110; I++)
		{ delayMicroseconds(100); if ((PIND&0x80)!=0) goto IRWait_1; }
	return 0;
IRWait_1:
	if (I<70) return 0;
	for (I=0; I<55; I++)
		{ delayMicroseconds(100); if ((PIND&0x80)==0) goto IRWait_2; }
	return 0;
IRWait_2:
	if (I<15) return 0;
	if (I<35) { delay(2); return 2; } else return 1;
}
//######################################################################################################################## 
byte MP1051Class::IRBit()
{	//	 
	byte I;
	for (I=0; I<8; I++)		//	 400...900us
		{ delayMicroseconds(100); if ((PIND&0x80)!=0) goto IRBit_0; }
	return 2;			//	  
IRBit_0:
	if (I<4) return 3;		//	  
	for (I=0; I<20; I++)		//	   200...2100us
		{ delayMicroseconds(100); if ((PIND&0x80)==0) goto IRBit_1; }
	return 4;			//	    , ?
IRBit_1:
	if (I<3) return 5;		//	    
	if (I>9) return 1;		//	 1
		else return 0;		//	 0
}
//######################################################################################################################## 
byte MP1051Class::IRRead()
{	//   4 
	//0 - ,  1
	byte BYT, BIT, I;
	BYT=0;
	for (I=0; I<8; I++)
		{
		BIT=IRBit();
		if (BIT==0) BYT=BYT<<1;
		if (BIT==1) BYT=(BYT<<1)|0x01;
		if (BIT>=2) return 0;
		}
	IR_ADR=BYT;
	BYT=0;
	for (I=0; I<8; I++)
		{
		BIT=IRBit();
		if (BIT==0) BYT=BYT<<1;
		if (BIT==1) BYT=(BYT<<1)|0x01;
		if (BIT>=2) return 0;
		}
	if (IR_ADR!=(0xFF-BYT)) return 0;
	BYT=0;
	for (I=0; I<8; I++)
		{
		BIT=IRBit();
		if (BIT==0) BYT=BYT<<1;
		if (BIT==1) BYT=(BYT<<1)|0x01;
		if (BIT>=2) return 0;
		}
	IR_DATA=BYT;
	BYT=0;
	for (I=0; I<8; I++)
		{
		BIT=IRBit();
		if (BIT==0) BYT=BYT<<1;
		if (BIT==1) BYT=(BYT<<1)|0x01;
		if (BIT>=2) return 0;
		}
	if (IR_DATA!=(0xFF-BYT)) return 0;
	delay(3);
	return 1;
}
//######################################################################################################################## 
byte MP1051Class::IR(int T)
{	//	 -
	//	0 - ,  1/2
	//	1 - ,  
	//	2 - 
	//	  (hex, ==0):
	//	A2	62	E2
	//	22	02	C2
	//	E0	A8	90
	//	68	98	B0
	//	30	18	7A
	//	10	38	5A
	//	42	4A	52
	byte RESULT;
	RESULT=IRWait(T);
	if (RESULT==0) { IR_ADR=0xFF; IR_DATA=0xFF; return 0; }
	if (RESULT==2) return 2;
	RESULT=IRRead();
	if (RESULT==0) { IR_ADR=0xFF; IR_DATA=0xFF; return 0; }
		else return 1;
}
//######################################################################################################################## 
byte MP1051Class::IRAdr()
{
	return IR_ADR;
}
//######################################################################################################################## 
byte MP1051Class::IRData()
{
	return IR_DATA;
}
//######################################################################################################################## 
ISR(TIMER2_OVF_vect)
{	//   
 	TCNT2=135;
	byte L=(ACNT>>5);
	byte B=(ACNT&0x1F);
	if (B==0)
		{
		PORTC|=0x0F;
		byte LD1=LED[L][0];
		byte LD2=LED[L][1];
		for(byte I=0; I<8; I++)
			{
			if ((LD1&0x80)==0) PORTC&=0b11101111; else PORTC|=0b00010000;
			if ((LD2&0x80)==0) PORTC&=0b11011111; else PORTC|=0b00100000;  
			LD1=LD1<<1;  LD2=LD2<<1;
			PORTD|=0b00000100;  //  CLK
			PORTD&=0b11111011;  //  CLK
			}
		PORTD|=0b00001000;  //  LE
		PORTD&=0b11110111;  //  LE
		if (L==0) PORTC&=0b11111110;
		if (L==1) PORTC&=0b11111101;
		if (L==2) PORTC&=0b11111011;
		if (L==3) PORTC&=0b11110111;
		}
	if (B<BRGH) PORTD&=0b11101111; else PORTD|=0b00010000;
	if (ACNT<127) ACNT++; else ACNT=0;
}
//######################################################################################################################## 